/**
 * $RCSfile: editor_plugin_src.js,v $
 * $Revision: 1.0 $
 * $Date: 2006/05/16 21:05:00 $
 *
 * @author Xele
 */

/* Import plugin specific language pack */
tinyMCE.importPluginLanguagePack('phpExec', 'en'); // <- Add a comma separated list of all supported languages

// Singleton class
var TinyMCE_PhpExecPlugin = {
	/**
	 * Returns information about the plugin as a name/value array.
	 * The current keys are longname, author, authorurl, infourl and version.
	 *
	 * @returns Name/value array containing information about the plugin.
	 * @type Array 
	 */
	getInfo : function() {
		return {
			longname : 'PhpExec in TinyMCE plugin',
			author : 'Xele',
			authorurl : 'http://www.xele.org',
			infourl : 'http://www.xele.org/2006/05/17/utiliser-phpexec-avec-tinymce/',
			version : "1.0"
		};
	},

	/**
	 * Gets executed when a TinyMCE editor instance is initialized.
	 *
	 * @param {TinyMCE_Control} Initialized TinyMCE editor control instance. 
	 */
	initInstance : function(inst) {
		tinyMCE.importCSS(inst.getDoc(), tinyMCE.baseURL + "/plugins/phpExec/css/phpExec.css");
	},

	/**
	 * Returns the HTML code for a specific control or empty string if this plugin doesn't have that control.
	 * A control can be a button, select list or any other HTML item to present in the TinyMCE user interface.
	 * The variable {$editor_id} will be replaced with the current editor instance id and {$pluginurl} will be replaced
	 * with the URL of the plugin. Language variables such as {$lang_somekey} will also be replaced with contents from
	 * the language packs.
	 *
	 * @param {string} cn Editor control/button name to get HTML for.
	 * @return HTML code for a specific control or empty string.
	 * @type string
	 */
	getControlHTML : function(cn) {
		switch (cn) {
			case "phpExec":
				return tinyMCE.getButtonHTML(cn, 'lang_phpExec_desc', '{$pluginurl}/images/phpExec.gif', 'mcePhpExec', true);
		}

		return "";
	},

	/**
	 * Executes a specific command, this function handles plugin commands.
	 *
	 * @param {string} editor_id TinyMCE editor instance id that issued the command.
	 * @param {HTMLElement} element Body or root element for the editor instance.
	 * @param {string} command Command name to be executed.
	 * @param {string} user_interface True/false if a user interface should be presented.
	 * @param {mixed} value Custom value argument, can be anything.
	 * @return true/false if the command was executed by this plugin or not.
	 * @type
	 */
	execCommand : function(editor_id, element, command, user_interface, value) {
		// Handle commands
		switch (command) {
			// Remember to have the "mce" prefix for commands so they don't intersect with built in ones in the browser.
			case "mcePhpExec":
				// Show UI/Popup
				if (user_interface) {
					// Open a popup window and send in some custom data in a window argument
					var phpExec = new Array();

					phpExec['file'] = '../../plugins/phpExec/popup.htm'; // Relative to theme
					phpExec['width'] = 300;
					phpExec['height'] = 200;

					tinyMCE.openWindow(phpExec, {editor_id : editor_id, php_code : ""});

					// Let TinyMCE know that something was modified
					tinyMCE.triggerNodeChange(false);
				} else {
					// Do a command this gets called from the phpExec popup
					//alert("execCommand: mcePhpExec gets called from popup.");
					var html = TinyMCE_PhpExecPlugin._makeImg(value);
					tinyMCE.execCommand("mceInsertContent",true,html);
				}

				return true;
		}

		// Pass to next handler in chain
		return false;
	},

	/**
	 * Gets called ones the cursor/selection in a TinyMCE instance changes. This is useful to enable/disable
	 * button controls depending on where the user are and what they have selected. This method gets executed
	 * alot and should be as performance tuned as possible.
	 *
	 * @param {string} editor_id TinyMCE editor instance id that was changed.
	 * @param {HTMLNode} node Current node location, where the cursor is in the DOM tree.
	 * @param {int} undo_index The current undo index, if this is -1 custom undo/redo is disabled.
	 * @param {int} undo_levels The current undo levels, if this is -1 custom undo/redo is disabled.
	 * @param {boolean} visual_aid Is visual aids enabled/disabled ex: dotted lines on tables.
	 * @param {boolean} any_selection Is there any selection at all or is there only a cursor.
	 */
	handleNodeChange : function(editor_id, node, undo_index, undo_levels, visual_aid, any_selection) {
		// Select phpExec button if parent node is a strong or b
		if (node.nodeName.toLowerCase() == "img" && tinyMCE.getAttrib(node, 'class').indexOf('phpExec') != -1) {
			tinyMCE.switchClass(editor_id + '_phpExec', 'mceButtonSelected');
			return true;
		}

		// Deselect phpExec button
		tinyMCE.switchClass(editor_id + '_phpExec', 'mceButtonNormal');
	},


	/**
	 * Gets called when HTML contents is inserted or retrived from a TinyMCE editor instance.
	 * The type parameter contains what type of event that was performed and what format the content is in.
	 * Possible valuses for type is get_from_editor, insert_to_editor, get_from_editor_dom, insert_to_editor_dom.
	 *
	 * @param {string} type Cleanup event type.
	 * @param {mixed} content Editor contents that gets inserted/extracted can be a string or DOM element.
	 * @param {TinyMCE_Control} inst TinyMCE editor instance control that performes the cleanup.
	 * @return New content or the input content depending on action.
	 * @type string
	 */
	cleanup : function(type, content, inst) {
		switch (type) {
			case "get_from_editor":
// 				alert("[FROM] Value HTML string: " + content);
				var startPos = -1;
				while ((startPos = content.indexOf('<img', startPos+1)) != -1) {
					var endPos = content.indexOf('/>', startPos);
					var attributes = TinyMCE_PhpExecPlugin._parseAttributes(content.substring(startPos + 4, endPos));

					if (attributes['class'] == "mce_plugin_phpExec") {
						endPos += 2;

						var embedHTML = '<phpcode>';
						if( attributes['alt'] != null && attributes['alt'] != '' )
							embedHTML += unescape(attributes['alt']);
						embedHTML += '</phpcode>';

						// Insert embed/object chunk
						chunkBefore = content.substring(0, startPos);
						chunkAfter = content.substring(endPos);
						content = chunkBefore + embedHTML + chunkAfter;
					}
				}
				// Do custom cleanup code here

				break;

			case "insert_to_editor":
				// Do custom cleanup code here
				startPos = 0;

				while ((startPos = content.indexOf('<phpcode>', startPos)) != -1) {
					// Insert image
					var endPos = content.indexOf('</phpcode>', startPos);
					var contentAfter = content.substring(endPos+10);
					var value = content.substring(startPos+9, endPos);
					content = content.substring(0, startPos);
					//alert('Value : '+value+'- Before : '+content+'- After'+contentAfter);
					content += TinyMCE_PhpExecPlugin._makeImg(value);
					content += contentAfter;

					startPos++;
				}
				break;

			case "get_from_editor_dom":
// 				alert("[FROM] Value DOM Element " + content.innerHTML);

				// Do custom cleanup code here

				break;

			case "insert_to_editor_dom":
// 				alert("[TO] Value DOM Element: " + content.innerHTML);

				// Do custom cleanup code here

				break;
		}

		return content;
	},

	// Private plugin internal methods
	_parseAttributes : function(attribute_string) {
		var attributeName = "";
		var attributeValue = "";
		var withInName;
		var withInValue;
		var attributes = new Array();
		var whiteSpaceRegExp = new RegExp('^[ \n\r\t]+', 'g');

		if (attribute_string == null || attribute_string.length < 2)
			return null;

		withInName = withInValue = false;

		for (var i=0; i<attribute_string.length; i++) {
			var chr = attribute_string.charAt(i);

			if ((chr == '"' || chr == "'") && !withInValue)
				withInValue = true;
			else if ((chr == '"' || chr == "'") && withInValue) {
				withInValue = false;

				var pos = attributeName.lastIndexOf(' ');
				if (pos != -1)
					attributeName = attributeName.substring(pos+1);

				attributes[attributeName.toLowerCase()] = attributeValue.substring(1);

				attributeName = "";
				attributeValue = "";
			} else if (!whiteSpaceRegExp.test(chr) && !withInName && !withInValue)
				withInName = true;

			if (chr == '=' && withInName)
				withInName = false;

			if (withInName)
				attributeName += chr;

			if (withInValue)
				attributeValue += chr;
		}

		return attributes;
	},

	_makeImg : function(value)
	{
		return '<img src="' + (tinyMCE.getParam("theme_href") + "/images/spacer.gif") + '" '
							+ ' width="80" height="16" '
							+ 'alt="'+escape(value)+'" title="" class="mce_plugin_phpExec" name="mce_plugin_phpExec" />';
	},

};

// Adds the plugin class to the list of available TinyMCE plugins
tinyMCE.addPlugin("phpExec", TinyMCE_PhpExecPlugin);
